==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/time.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: time.c,v 1.12 2004/06/25 12:29:02 lwa Exp $";

#include "config.h"

#define _REENTRANT
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include "lmtp.h"

#if !HAVE_SETENV
/* use fake function for Solaris */
static int setenv(char *envname, char *envval, int overwrite) {
  int len = strlen(envname) + strlen(envval) + 2;
  char *string = malloc(len);
  if (string) {
    sprintf(string, "%s=%s", envname, envval);
    putenv(string);
    free(string);
    return 0;
  } else {
    return -1;
  }
}
#endif

long a2delay(char *a) {
  if (a) {
    long delay = strtol(a, &a, 10);
    if (a) {
      switch(*a) {
      case 'm': case 'M': delay *= 60; break;
      case 'h': case 'H': delay *= 60*60; break;
      case 'd': case 'D': delay *= 60*60*24; break;
      case 'w': case 'W': delay *= 60*60*24*7; break;
      }
    }
    return delay;
  }
  return 0;
}

void time_init(struct fstate *state) {
  if (state->timezone)
    setenv("TZ", state->timezone, 1);
  else {
#if HAVE_UNSETENV
    unsetenv("TZ");
#else
    setenv("TZ", "", 1);
#endif    
  }
  tzset();
}

struct tm *usertime(struct fstate *state) {
  static struct tm xtm;
  if (state->timezone_user) {
    setenv("TZ", state->timezone_user, 1);
    tzset();
  }
  localtime_r(&state->date, &xtm);
#if HAVE_STRUCT_TM_TM_ZONE
  if (xtm.tm_zone) {
    static char tm_zone[10];
    strncpy(tm_zone, xtm.tm_zone, sizeof tm_zone);
    tm_zone[sizeof tm_zone-1]=0;
    xtm.tm_zone=tm_zone;
  }
#endif
  if (state->timezone_user)
    time_init(state);
  return &xtm;
}

score_t scoretime(struct fstate *state, char **buf) {
  char *rest = (*buf)+1;
  struct tm *xtm=usertime(state);
  score_t score=0;
  int stop=0;

  while(*rest && !stop) {
    switch(*rest) {
    case 'c': case 'C': score=score*100+(xtm->tm_year+1900)/100; rest++; break;
    case 'y': case 'Y': score=score*100+xtm->tm_year%100; rest++; break;
    case 'm': case 'M': score=score*100+xtm->tm_mon+1; rest++; break;
    case 'd': case 'D': score=score*100+xtm->tm_mday; rest++; break;
    case 'w': case 'W': score=score*10+xtm->tm_wday; rest++; break;
    case 'h': case 'H': score=score*100+xtm->tm_hour; rest++; break;
    case 'n': case 'N': score=score*100+xtm->tm_min; rest++; break;
    case 's': case 'S': score=score*100+xtm->tm_sec; rest++; break;
    default:
      stop=1;
    }
    if (score <0)
      score=-1;
  }
  *buf=rest;
  return score;
}

==> lmtpd-0.10.7.tar.gz: lmtpd-0.10.7/time.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: time.c,v 1.13 2004/10/26 09:59:20 lwa Exp $";



#define _REENTRANT
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include "lmtp.h"

#if !HAVE_SETENV
/* use fake function for Solaris */
static int setenv(char *envname, char *envval, int overwrite) {
  int len = strlen(envname) + strlen(envval) + 2;
  char *string = malloc(len);
  if (string) {
    sprintf(string, "%s=%s", envname, envval);
    putenv(string);
    free(string);
    return 0;
  } else {
    return -1;
  }
}
#endif

long a2delay(char *a) {
  if (a) {
    long delay = strtol(a, &a, 10);
    if (a) {
      switch(*a) {
      case 'm': case 'M': delay *= 60; break;
      case 'h': case 'H': delay *= 60*60; break;
      case 'd': case 'D': delay *= 60*60*24; break;
      case 'w': case 'W': delay *= 60*60*24*7; break;
      }
    }
    return delay;
  }
  return 0;
}

void time_init(struct fstate *state) {
  if (state->timezone)
    setenv("TZ", state->timezone, 1);
  else {
#if HAVE_UNSETENV
    unsetenv("TZ");
#else
    setenv("TZ", "", 1);
#endif    
  }
  tzset();
}

struct tm *usertime(struct fstate *state) {
  static struct tm xtm;
  if (state->timezone_user) {
    setenv("TZ", state->timezone_user, 1);
    tzset();
  }
  localtime_r(&state->date, &xtm);
#if HAVE_STRUCT_TM_TM_ZONE
  if (xtm.tm_zone) {
    static char tm_zone[10];
    strncpy(tm_zone, xtm.tm_zone, sizeof tm_zone);
    tm_zone[sizeof tm_zone-1]=0;
    xtm.tm_zone=tm_zone;
  }
#endif
  if (state->timezone_user)
    time_init(state);
  return &xtm;
}

score_t scoretime(struct fstate *state, char **buf) {
  char *rest = (*buf)+1;
  struct tm *xtm=usertime(state);
  score_t score=0;
  int stop=0;

  while(*rest && !stop) {
    switch(*rest) {
    case 'c': case 'C': score=score*100+(xtm->tm_year+1900)/100; rest++; break;
    case 'y': case 'Y': score=score*100+xtm->tm_year%100; rest++; break;
    case 'm': case 'M': score=score*100+xtm->tm_mon+1; rest++; break;
    case 'd': case 'D': score=score*100+xtm->tm_mday; rest++; break;
    case 'w': case 'W': score=score*10+xtm->tm_wday; rest++; break;
    case 'h': case 'H': score=score*100+xtm->tm_hour; rest++; break;
    case 'n': case 'N': score=score*100+xtm->tm_min; rest++; break;
    case 's': case 'S': score=score*100+xtm->tm_sec; rest++; break;
    default:
      stop=1;
    }
    if (score <0)
      score=-1;
  }
  *buf=rest;
  return score;
}
